effects = {}

effects.playerJump = {}
function effects.playerJump:initialize(loc)		
	local e = {	
		chance = 75,
		x = loc.x + (loc.w/2),
		y = loc.y + (loc.h/2),
		xv = loc.w/2,
		yv = loc.h/3,
		rw = loc.w,
		w = 3,
		wv = 2,
		h = 3,
		hv = 2,	
		particleAmount = 1,
		particleVariance = 1,
		seconds = 0.08,
		alpha = 255,
		fade = 0,
		color = colorConstants.orange,
		gravity = 200,
		queueSpot = 0,
	}
	function e:update(dt, el)
		if self.seconds > 0 then
			self.y = self.y + (self.gravity*dt)
			self.seconds = self.seconds - dt
			self.alpha = self.alpha - self.fade
		else
			self:destroy(el)
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color, self.alpha)		
		local p = self.particleAmount + math.random(0, self.particleVariance)
		for i=1, p do
			local r = self.hv * (math.random(0,self.wv))
			love.graphics.rectangle('line', 
			(self.x + ( math.random(-self.xv,self.xv))), 
			(self.y + math.random(-self.yv,self.yv)), 
			(self.w + r), 
			(self.h + r))
		end
	end

	return e
end
effects.fire = {}
function effects.fire:initialize(loc, sub)		
	if sub == nil then sub = 1 end
	local subHalf
	if sub/2 < 1 then subHalf = 1
	else subHalf = sub/2 end
	--for fire, sub is a multiplier to size
	local e = {	
		x = loc.x + (loc.w/2),
		y = loc.y + (loc.h/2),
		xv = 20 * sub,
		yv = 5 * sub,
		rw = loc.w,
		w = 1.5,
		wv = 1 * sub,
		h = 1.5,
		hv = 1 * sub,	
		segs = 10,
		particleAmount = 1,
		particleVariance = 0,
		seconds = 0.4 * subHalf,
		alpha = 255,
		fade = 255,
		color = colorConstants.red,
		gravity = -150 * subHalf,
		queueSpot = 0,
	}
	function e:update(dt, el)
		if self.seconds > 0 then
			self.y = self.y + (self.gravity*dt)
			self.seconds = self.seconds - dt
			self.alpha = self.alpha - self.fade * (dt/self.seconds)
		else
			self:destroy(el)
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color, self.alpha)		
		local p = self.particleAmount + math.random(0, self.particleVariance)
		for i=1, p do
			local r = self.hv * (math.random(0,self.wv))
			love.graphics.circle('fill', 
			(self.x + ( math.random(-self.xv,self.xv))), 
			(self.y + math.random(-self.yv,self.yv)), 
			(self.w + r), 
			self.segs)
		end
	end

	return e
end

effects.enemyDeath = {}

function effects.enemyDeath:initialize(loc, sub)		
	if sub == nil then sub = 1 end
	local subHalf
	if sub/2 < 1 then subHalf = 1
	else subHalf = sub/2 end
	--for fire, sub is a multiplier to size
	local e = {	
		x = loc.x + (loc.w/2),
		y = loc.y + (loc.h/2),
		xv = 20 * sub,
		yv = 20 * sub,
		rw = loc.w,
		w = 1.5,
		wv = 1 * sub,
		h = 1.5,
		hv = 1 * sub,	
		segs = 10,
		particleAmount = 1,
		particleVariance = 2,
		seconds = 0.4 * subHalf,
		alpha = 255,
		fade = 255,
		color = loc.color,
		gravity = 0,
		queueSpot = 0,
	}
	function e:update(dt, el)
		if self.seconds > 0 then
			self.seconds = self.seconds - dt
			self.alpha = self.alpha - self.fade * (dt/self.seconds)
		else
			self:destroy(el)
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color, self.alpha)		
		local p = self.particleAmount + math.random(0, self.particleVariance)
		for i=1, p do
			local r = self.hv * (math.random(0,self.wv))
			love.graphics.circle('fill', 
			(self.x + ( math.random(-self.xv,self.xv))), 
			(self.y + math.random(-self.yv,self.yv)), 
			(self.w + r), 
			self.segs)
		end
	end

	return e
end


effects.attackParticle = {}
function effects.attackParticle:initialize(loc)
	local e = {	
		x = loc.x + math.random(0, loc.w),
		y = loc.y + math.random(-loc.h, loc.h),
		r = (math.random(0.1, 0.15)) * 2,
		rint = 15,
		seconds = 0.05 + math.random(0.1, 0.2),
		alpha = 255,
		color = colorConstants.green,
		queueSpot = 0,
		segs = 8,
	}
	function e:update(dt, el)
		if self.seconds > 0 then
			self.r = self.r + self.rint * dt
			self.seconds = self.seconds - dt
		else
			self:destroy(el)
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color, self.alpha)
		love.graphics.circle('line',self.x,self.y,self.r,self.segs)
	end

	return e
end

effects.heal = {}
function effects.heal:initialize(loc)
	local e = {	
		x = loc.x + math.random(-loc.w/2, loc.w),
		y = loc.y - math.random(loc.h/2, loc.h),
		h = loc.w/4,
		w = 1,
		speed = 600,
		seconds = 0.1,
		alpha = 255,
		color = colorConstants.white,
		queueSpot = 0,
	}
	function e:update(dt, el)
		if self.seconds > 0 then
			self.y = self.y - self.speed * dt
			self.seconds = self.seconds - dt
			self.alpha = self.alpha - 255 * self.seconds * dt
		else
			self:destroy(el)
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color, self.alpha)
		love.graphics.rectangle('fill',self.x,self.y,self.w,self.h)
	end

	return e
end

effects.sparkle = {}
function effects.sparkle:initialize(loc)	
	local mod = math.random(0,2)
	local e = {	
		x = (map.w * map.block.w) / 2,
		y = (map.h * map.block.h) /2,
		r = 1 + mod,
		segs = 7,
		speedmod = 40,
		moveamount = 0,
		xvel = 0,
		yvel = 0,
		xdest = 0,
		ydest = 0,
		color = colorConstants.cyan,
		alpha = 100,
		queueSpot = 0,
		distanceclamp = map.w * map.block.w - 200,
	}
	e.originx = e.x
	e.originy = e.y
	function e:update(dt, el)
		if self.moveamount > 0 then
			self.moveamount = self.moveamount - dt
		else			
			self.moveamount = math.random(5,30)/100
			self.xdest = math.random(-100,100)
			if self.xdest ~= 0 then self.xdest = self.xdest/100
			end
			self.ydest = math.random(-100,100)
			if self.ydest ~= 0 then self.ydest = self.ydest/100
			end
			
			self.xdest = self.x + self.xdest*self.speedmod
			self.ydest = self.y + self.ydest*self.speedmod
			
			self.xvel, self.yvel = getSpeedByVector(self.x,self.y, self.xdest, self.ydest)
		end
		self.x = self.x + self.xvel * dt * self.speedmod
		self.y = self.y + self.yvel * dt * self.speedmod
		
		if self.x < self.originx - self.distanceclamp or self.x > self.originx + self.distanceclamp then
			self.x = self.x - self.xvel * dt * self.speedmod
		end
		if self.y < self.originy - self.distanceclamp or self.y > self.originy + self.distanceclamp then
			self.y = self.y - self.yvel * dt * self.speedmod
		end
	end
	function e:destroy(el)
		el.garbage = el.garbage + 1
		el[self.queueSpot] = nil
	end
	function e:draw()
		love.graphics.setColor(self.color,self.alpha)
		love.graphics.circle('line',self.x,self.y,self.r,self.segs)	
	end

	return e
end

return effects