player = {
		--general properties. may perhaps be categorized in later practices
		x = 110,
		xBase = 110,
		y = 300,
		yBase = 300,
		hp = 45,
		hpBase = 45,
		hpMax = 60,
		w = 16,
		h = 32,
		s = 450,
		d = 10,
		iframes = 0,
		name = "player",
		index = "player",
		sfx = {attack = {f = "player_attack.ogg", t = "static", v = 0.7,}, --path starts from "assets/sounds"
			jump = {f = "player_jump.ogg", t = "static", v = 0.55,},
			pain = {f = "player_pain.ogg", t = "static", v = 0.4,},			
		},
		
		--subcategories
			external = { --vector impacts
				state = 'none', --also 'partial' and 'full'
				updates = 0,
				xd = 0,
				yd = 0,
				speedMod = 0,
			},
			input = {
				jumpButton = false,
				jumpButtonReleased = true,
				direction = "right",
				xvel = 0,
				yvel = 0,
				},
			air = { 
				state = "falling",
				jump = {--player.air.jump
						speed = 650, -- start at this @ jump and decrement by gravityResist
						yvel = 500,
						jumpsLeft = 0,	-- how many jumps we can perform. can only perform jumps while !jumping, and resets to jumpsMax when hit ground
						jumpsMax = 0x7FFFFFFF,	-- resets jumpsLeft to this when hit ground
						jumpHoldUpdates = 0,
						jumpHoldUpdatesMax = 3,
						jumpHeight = 80,
						jumpIncrease = 30,
						jumpGoal = 0,
						fallDist = 25,
						fallGoal = 0,
						},
				gravityMax = 1500,
				gravityStart = 300,
				gravity = 700,
				gravityResist = 600,
				jumpDT = 0
				},					
			drawStuff = {
				drawState = true,
				mode = "line",
				color = colorConstants.bright_green,
				alpha = 255,
				dirOffset = 0,
				dir = 3,
				faceX = 3.5,
				faceX2 = 3.5,
				faceY = 1.5,
				faceY2 = 13.5,
				faceState = {"o", "--"},
				faceHappy = {"o", "--"},
				facePain = {"x","0"},
				faceUpdatesLeft = 0,
				},
			damage = {
				damaging = false,
				otherx = 0,
				othery = 0,
				otherd = 0,	
				},
			attack = {
				w = 1,
				h = 10,
				x = 0,
				y = 0,
				button = false,
				attacking = false,
				d = 10,
				dur = 1, -- seconds
				length = 60,
				curlen = 0,
				postdur = 1, -- seconds
				wait = 0,
				color = {[1] = colorConstants.bright_red, [2] = colorConstants.bright_orange,
						[3] = colorConstants.bright_yellow, [4] = colorConstants.bright_green,
						[5] = colorConstants.bright_blue, [6] = colorConstants.bright_purple},
				instance = {},
				name = "playerAttack",
			},
			debug = {
				},
}


function player:initialize()
	world:add(self,self.x,self.y,self.w,self.h)
end


function player:filter(other)
  local kind = other.name
  
  if kind == 'playerAttack' then
	return false
  elseif kind == 'bouncyblock' then
	return 'bounce'
  elseif kind == 'enemy' then
	if self.iframes > 0 then
		return false
	elseif other.state == 'dead' or other.state == 'dying' then
		return false
	else
		return 'slide'
	end
  else
	return 'slide'
  end
end

function player:attackfilter(other)
	if other.name == 'enemy' and other.state ~= 'dead' and other.state ~= 'dying' then other:setdamage() end
	
	return false
end


function player:update(dt)
	self:updateInvincibility(dt)
	self:updatePosition(dt)
	self:updateAttack(dt)	
	self:updateAnims(dt)
end

function player:updateInvincibility(dt)
	if self.iframes > 0 then
		self.iframes = self.iframes - dt
		if self.drawStuff.drawState then
			self.drawStuff.drawState = false
		else
			self.drawStuff.drawState = true
		end
	else
		self.drawStuff.drawState = true
	end
end

function player:updatePosition(dt)
	local px, py = self:resolveXInput(), self:resolveYInput(dt)
	--get x and y velocities attempted on player by outside
	local ox, oy = self:resolveXOutside(), self:resolveYOutside()
	--get goalX and goalY by applying outside v to player v
	local goalX, goalY = self:resolveVelocities(dt,px,py,ox,oy)
	local actualX, actualY, cols, len = world:move(self, goalX, goalY, self.filter)
	for i=1,len do
		local other = cols[i].other
		if other.name == "enemy" then 
			if other.state ~= 'dead' and other.state ~= 'dying' then
				local ox,oy = other:center()
				self:setdamage(ox,oy,other.d) 
			end
		else
			self:resolveCollisionByType(other, actualX, actualY)
		end
	end
	if self.air.state == 'jumping' or self.air.state == 'jumpFloating' then
		actualY = self:clampToHeightMax(actualY)
	end
	self.x, self.y = actualX, actualY
end

function player:resolveXInput() --this is fairly redundant, for now
	local result = 0
	result = self.input.xvel
	return result
end

function player:clampToHeightMax(goalY)
	local actualY
	local state = self.air.state
	if state == 'jumping' or state == 'jumpFloating' then
		if goalY < self.air.jump.jumpGoal then
			actualY = self.air.jump.jumpGoal
		else
			actualY = goalY
		end	
	end
	return actualY
end

function player:checkIfGroundOrCeiling()
	local grounded = false
	local ceiling = false
	local testY = math.floor(self.y + 1)
	currentY = math.floor(self.y)
	local actualX, actualY = world:check(self,self.x,testY,player.filter)
	actualY = math.floor(actualY)					
	if actualY == currentY then grounded = true end
	testY = math.floor(self.y - 1)
	actualX, actualY = world:check(self,self.x,testY,player.filter)
	if actualY == currentY then ceiling = true end
	return grounded, ceiling
end
	
function player:resolveYInput(dt)
	local jump = self.air.jump
	local gravity = self.air.gravity
	local gravityResist = self.air.gravityResist
	local gravityMax = self.air.gravityMax
	local input = self.input
	local state = self.air.state
	local y = self.y
	local isGrounded, isCeiling = self:checkIfGroundOrCeiling()
	local result	

	if state == 'jumping' then
		result = jump.yvel
		jump.yvel = jump.yvel + (gravityResist * dt)
		local stillClimbing = self:checkJumpHold()
		if isCeiling then
			self:airStateSwitch('falling')
		elseif not input.jumpButton or not stillClimbing then
			self:airStateSwitch('jumpFloating')
		end		
		effects:new("playerJump", self)
	elseif state == 'jumpFloating' then
		result = jump.yvel
		jump.yvel = jump.yvel + (gravityResist * dt)
		if input.jumpButton and input.jumpButtonReleased and jump.jumpsLeft > 0 then
			self:airStateSwitch('jumping')
		elseif isCeiling or y <= jump.jumpGoal then
			self:airStateSwitch('falling')
		end		
		effects:new("playerJump", self)
	elseif state == 'falling' then
		result = jump.yvel
		if jump.yvel < gravityMax then
			jump.yvel = jump.yvel + (gravity * dt)
		end
		if jump.yvel > gravityMax then
			jump.yvel = gravityMax
		end
		
			if input.jumpButton and input.jumpButtonReleased and jump.jumpsLeft > 0 and self.y > self.air.jump.fallGoal then
				self:airStateSwitch('jumping')
			elseif isGrounded then
				self:airStateSwitch('grounded')
			end
	elseif state == 'grounded' then
		result = 1
			if input.jumpButton and input.jumpButtonReleased then
				self:airStateSwitch('jumping')
			elseif isGrounded == false then
				self:airStateSwitch('falling')
			end
	else --state=='none'
		result = 0
	end
	
	return result
end

function player:checkJumpHold()
	local jump = self.air.jump
	local input = self.input
	local result	
	if input.jumpButton and jump.jumpHoldUpdates < jump.jumpHoldUpdatesMax then
		jump.jumpHoldUpdates = jump.jumpHoldUpdates + 1
		jump.jumpGoal = jump.jumpGoal - jump.jumpIncrease
		result = true
	else
		result = false
	end
	return result
end

function player:airStateSwitch(stateToSwitch)
	local p = stateToSwitch
	
	if p == 'jumping' then
		self.sfx.jump:play()
		self.air.state = 'jumping'
		self.air.jump.yvel = -self.air.jump.speed
		self.input.jumpButtonReleased = false
		self.air.jump.jumpsLeft = self.air.jump.jumpsLeft - 1
		self.air.jump.jumpGoal = self.y - self.air.jump.jumpHeight	
	elseif p == 'grounded' then
		self.air.state = 'grounded'
		self.air.jump.yvel = 0
		self.air.jump.jumpsLeft = self.air.jump.jumpsMax	
	elseif p == 'falling' then
		self.air.state = 'falling'
		self.air.jump.yvel = self.air.gravityStart
		self.air.jump.jumpHoldUpdates = 0	
		self.air.jump.fallGoal = self.y + self.air.jump.fallDist
	elseif p == 'jumpFloating' then
		self.air.state = 'jumpFloating'
		self.air.jump.jumpHoldUpdates = 0
	elseif p == 'none' then
		self.air.state = 'none'	
	end
end


function player:resolveXOutside()
	if self.external.state == 'none' then
		return 0
	elseif self.external.updates > 0 then
		self.external.updates = self.external.updates - 1
		return self.external.xd * self.external.speedMod
	else
		self:airStateSwitch('falling')
		self.external.state = 'none'
		return self.external.xd * self.external.speedMod
	end
end

function player:resolveYOutside()
	if self.external.state == 'none' then
		return 0
	elseif self.external.updates > 0 then
		self.external.updates = self.external.updates - 1
		return self.external.yd * self.external.speedMod
	else
		self:airStateSwitch('falling')
		self.external.state = 'none'
		return self.external.yd * self.external.speedMod
	end
end

function player:resolveVelocities(dt,px,py,ox,oy)
--ox,oy stand for 'outside x and y' and are not yet implemented
	if self.external.state == 'none' then				
		goalX = self.x + (px * dt)
		goalY = self.y + (py * dt)
	elseif self.external.state == 'partial' then
		goalX = self.x + (((px * 0.7) - ox) * dt)
		goalY = self.y + (((py * 0.5) - oy) * dt)
	elseif self.external.state == 'unbreakable' then
		goalX = self.x + (ox * dt)
		goalY = self.y + (oy * dt)
	end
	
return goalX, goalY
end

function player:resolveCollisionByType(col, goalX, goalY)
	if col.name == 'bouncyblock' then
		col.drawStuff.state = 'bounced'
		self.external.state = col.bounceType or 'partial'
		self.external.updates = col.bounceLen or 15
		self.external.speedMod = col.bounceSp or 20
		local tx,ty = col:center()
		local ox,oy = goalX, goalY
		self.external.xd, self.external.yd = getSpeedByVector(tx,ty,ox,oy)
		
		self:airStateSwitch('grounded')
		
	elseif col.name == 'damageblock' and self.iframes <= 0 then
		col.drawStuff.state = 'damaged'
		self:setdamage(col.x,col.y,col.d or 10) 
	elseif col.name == 'healblock' and not col.healcd and col.canHeal then
		col:healAnim()
		col.healcd = true
		col.healsLeft = col.healsLeft - 1
		if col.healsLeft < 1 then
			col.color = colorConstants.greyish_green
			col.canHeal = false
		end
		self:heal()
	end
end

function player:updateAttack(dt)
	if self.attack.button and not self.attack.attacking then
		self.attack.attacking = true
		local attack = self.attack
		attack.curlen = 1
		attack.wait = 0
		attack.wmod = self.w/2
		attack.hmod = self.h/2 - 4
		attack.y = self.y + self.attack.hmod
		if self.input.direction == 'right' then
			attack.x = self.x + attack.wmod
		else -- direction is left
			attack.x = self.x - attack.w + attack.wmod
		end
		
		world:add(attack, attack.x, attack.y, attack.w, attack.h)
		self.attack.instance = attack
		self.sfx.attack:play()
	elseif self.attack.attacking then
		self.attack.instance.w = self.attack.instance.curlen
		self.attack.instance.y = self.y + self.attack.instance.hmod
		if self.input.direction == 'right' then
			self.attack.instance.x = self.x + self.attack.instance.wmod
		else -- direction is left
			self.attack.instance.x = self.x - self.attack.instance.w + self.attack.instance.wmod
		end
		world:move(self.attack.instance,self.attack.instance.x,self.attack.instance.y,self.attackfilter)
		world:update(self.attack.instance,self.attack.instance.x,self.attack.instance.y,self.attack.instance.w,self.attack.instance.h)

		if self.attack.instance.curlen < self.attack.instance.length then
			self.attack.instance.curlen = self.attack.instance.curlen + 300 * dt
		elseif self.attack.instance.wait < self.attack.instance.postdur then
			self.attack.instance.wait = self.attack.instance.wait + (dt/self.attack.instance.postdur)
		else
			self.attack.attacking = false
			world:remove(self.attack.instance)
		end
	end
end

function player:drawAttack()
	a = self.attack.instance
	lg.setColor(a.color[math.random(1,#a.color)])
	love.graphics.rectangle('fill',a.x,a.y,a.w,a.h)
	effects:new("attackParticle",a)
end


function player:draw()	
	if self.drawStuff.drawState then
		if player.drawStuff.faceUpdatesLeft > 0 then
			player.drawStuff.faceUpdatesLeft = player.drawStuff.faceUpdatesLeft - 1
		else
			player.drawStuff.faceState = player.drawStuff.faceHappy
		end
		
		love.graphics.setColor(self.drawStuff.color, self.drawStuff.alpha)
		love.graphics.rectangle(self.drawStuff.mode, self.x, self.y, self.w, self.h)
		love.graphics.print(self.drawStuff.faceState[1],self.x + self.drawStuff.faceX + self.drawStuff.dirOffset,self.y + self.drawStuff.faceY)
		love.graphics.print(self.drawStuff.faceState[2],self.x + self.drawStuff.faceX2 + self.drawStuff.dirOffset,self.y + self.drawStuff.faceY2)
	end
	
	if self.attack.attacking then
		self:drawAttack()
	end
end

function player:setdamage(otherx, othery, otherd)
	if not self.damage.damaging then
		self.damage.damaging = true
		self.damage.otherx = otherx
		self.damage.othery = othery
		self.damage.otherd = otherd			
		damageList[damageList.number] = self
		damageList.number = damageList.number + 1
	end
	
end

function player:resolveDamage()
	otherx = self.damage.otherx
	othery = self.damage.othery
	damage = self.damage.otherd
	if self.iframes <= 0 then
		if self.external.state == 'none' or self.external.state == 'partial' then		
			self.external.state = "unbreakable"
			self.external.updates = 7
			local x,y = self:center()			
			self.external.xd, self.external.yd = getSpeedByVector(x,y,otherx,othery)
			self.external.speedMod = 18
			self.drawStuff.faceState = self.drawStuff.facePain
			self.drawStuff.faceUpdatesLeft = 5
			self.iframes = 1.3
			self:airStateSwitch('none')
			self.sfx.pain:play()
		end
	end
	self.hp = self.hp - 5
	
	self.damage.damaging = false
end

function player:heal()
	if self.hp < self.hpMax then
		self.hp = self.hp + 2
	end
	self.healed = true
	self.healTimer = 0.5
	self.healInc = 0.01
end

function player:updateAnims(dt)
	if self.healed then
		if self.healTimer > 0 then
			self.healTimer = self.healTimer - dt
			if self.healInc > 0 then
				self.healInc = self.healInc - dt
			else
				effects:new("heal",self)
				self.healInc = 0.01
			end
		end
	end
end

function player:center()
	local cx, cy = self.x, self.y
	cx = cx + (self.w/2)
	cy = cy + (self.h/2)
	return cx, cy
end

return player