foe = {}

function foe:initialize(mod)
	local enemy = {
		index = "enemy",
		exists = true,
		alive = true,
		goalX = 0,
		goalY = 0,
		w = 32,
		h = 32,
		hvel = 0,
		vvel = 0,
		hp = 1,
		d = 10, --damage
		actionUpdates = 13,
		updateAmount = 0,
		updates = 0,
		speed = 200,
		state = "init",
		color = colorConstants.red,
		font = colorConstants.white,
		lineColor = colorConstants.blue,
		linewidth = 2,
		faceOffsetX = 3,
		faceOffsetY = 4,
		name = "enemy",
		index = "enemy",
		sfx = {
			death = {f = "robot_death.ogg", t = "static", v = 0.3,},			
		},
		death = {
				secondsToDie = 0.5,
				yspeed = 240,
				gravity = 240,
				},
		damage = {
				damaging = false,
				otherx = 0,
				othery = 0,
				otherd = 0,	
				},
		}

	function enemy:update(dt,i)
		state = self.state
		if state == "dying" then
			if self.death.secondsToDie > 0 then
				self.death.secondsToDie = self.death.secondsToDie - dt
			else
				self:kill()
			end			
		elseif state == "dead" then
		else 
			local hr = math.random(-1,1)
			local vr = math.random(-1,1)
			local ur = math.random(0,10)
			if state == "init" then			
				self:goDirection(hr,vr,ur)
				self.state = "going"
			elseif state == "going" then
				self:continueGoing(dt)
			elseif state == "stopped" then
				self:goDirection(hr,vr,ur)
				self.state = "going"
			end
		end
		if self.state ~= 'dead' and self.state ~= 'dying' then
			local actualX, actualY, cols, len = world:move(self, self.goalX, self.goalY, self.filter)
			self.x, self.y = actualX, actualY
			for i=1,len do
				local other = cols[i].other
				if other.name == "playerAttack" then 
					self:setdamage(other.x, other.y, other.d or 10)
				elseif other.name == "player" then
					local x,y = self:center()
					other:setdamage(x,y,self.d) 
				end
			end
		end
	end
	function enemy:setdamage(otherx, othery, otherd)
		if not self.damage.damaging then
			self.damage.damaging = true
			self.damage.otherx = otherx
			self.damage.othery = othery
			self.damage.otherd = otherd			
			damageList[damageList.number] = self
			damageList.number = damageList.number + 1
		end
	end
	function enemy:resolveDamage()
		effects:new("enemyDeath",self, 3)
		self.sfx.death:play()
		self.state = 'dying'
	end
	function enemy:goDirection(h,v,u)
		self.updates = self.actionUpdates + u
		self.hvel = h * self.speed * self.sMod
		self.vvel = v * self.speed * self.sMod
		self.updateAmount = self.updateAmount + 1
	end
	function enemy:continueGoing(dt)
		if self.updates > 0 then
			self.goalX = self.x + (self.hvel*dt)
			self.goalY = self.y + (self.vvel*dt)
			self.updates = self.updates - 1
		else
			self.state = "stopped"
		end
	end
	function enemy:draw()
		if self.state and self.state ~= 'dead' and self.alive then
			love.graphics.setColor(self.lineColor,255)
			love.graphics.rectangle('line',self.x,self.y,self.w,self.h)
			love.graphics.setColor(self.color,255)
			love.graphics.rectangle('fill',self.x+self.linewidth,self.y+self.linewidth,self.w-self.linewidth,self.h-self.linewidth)
			love.graphics.setColor(self.font,255)
			if self.state == "dying" then
				love.graphics.print("X__X",self.x+self.faceOffsetX,self.y+self.faceOffsetY)
			else
				love.graphics.print("  V ",self.x+self.faceOffsetX,self.y + (self.faceOffsetY - 2))
				love.graphics.print("O_O",self.x+self.faceOffsetX,self.y+self.faceOffsetY)
			end	
		end
	end
	function enemy:center()
		local cx, cy = self.x + (self.w/2), self.y + (self.h/2)
		return cx, cy
	end
	function enemy:kill()
		enemyRNG.enemyCount = enemyRNG.enemyCount - 1
		world:remove(self)
		self.state = 'dead'
		self.alive = false
	end
	function enemy:filter(other)
		if self.state == 'dead' then
			return false
		elseif self.state == 'dying' then
			if kind == 'player' or kind == 'playerAttack' then
				return false
			else
				return 'slide'
			end
		else
			local kind = other.name
			if kind == 'player' then
				if other.iframes > 0 then
					return false
				else
					return 'slide'
				end
			elseif kind == 'playerAttack' then
				return false
			else
				return 'slide'
			end
		end
	end
	
	return enemy
end

	

e_red = {}
function e_red:initialize(x,y, mod)
	local e = {}
	e = foe:initialize()
	e.x = x
	e.y = y
	e.color = colorConstants.red
	e.font = colorConstants.white
	e.sMod = 2 + mod.speed
	return e
end
e_blue = {}
function e_blue:initialize(x,y, mod)
	local e = {}
	e = foe:initialize()
	e.x = x
	e.y = y
	e.color = colorConstants.blue
	e.font = colorConstants.cyan
	e.sMod = 1 + mod.speed
	return e
end
e_orange = {}
function e_orange:initialize(x,y, mod)
	local e = {}
	e = foe:initialize()
	e.x = x
	e.y = y
	e.color = colorConstants.orange
	e.font = colorConstants.black
	e.sMod = 1 + mod.speed
	return e
end
e_yellow = {}
function e_yellow:initialize(x,y, mod)
	local e = {}
	e = foe:initialize()
	e.x = x
	e.y = y
	e.color = colorConstants.yellow
	e.font = colorConstants.black
	e.sMod = 0.5 + mod.speed
	return e
end
