local t = require 'engine.gameplay.transition'
local pau = require 'engine.gameplay.pause'
local pl = require 'engine.gameplay.play'
local win = require 'engine.gameplay.win'
local lose = require 'engine.gameplay.lose'

gameState = {
	index = "gameState",
	--we shouldn't need skipdraw, because statechange happens before state.update(dt)
	audioSystem = {},
	state = "", --transition, pause, play
	transition = t,
	pause = pau,
	play = pl,
	win = win,
	lose = lose,
	gameModifiers = {
		speed = 1,
		enemies = 10,
		heals = 0,
	},
}
local a = {}
function gameState:load(game)
	a = self.audioSystem
	if game == "new" then
		a:loadProfile(self.play)
		self.play:load(game, self.gameModifiers, a)
		self.state = "play"
	elseif game == "nextPlay" then
		self.play:load(game, self.gameModifiers)
		self.state = "play"
		--a:loadProfile(current game state) 
		--get audio stuff then in play.lua, can call audio stuff
	else --game will == a filepath to load from
	end
end

function gameState:update(dt)
	local s = self[self.state] --listen for statechange in child
	if s.stateChange ~= self.state then 
		print("updateNumber"..tostring(updateNumber).." o: "..self.state.." n: "..s.stateChange)
		self:stateSwitch(s.stateChange)
	end
	
	self[self.state]:update(dt, self.state, self.gameModifiers)
	self.skipDraw = false
end

function gameState:draw()
	if self.state == "transition" or self.state == "pause" then
		self.play:draw(self.state) --pass the other state, and play will have a modified draw
		self[self.state]:draw()

	else --play, win, and lose get drawn on their own
		self[self.state]:draw()
	end
end

function gameState:unload()
	--might give pause screen the "end game" option
	self.transition:unload()
	self.pause:unload()
	self.play:unload()
	--unload each state individually, because play doesn't unload when it switches
end

function gameState:stateSwitch(state) --only for switching between the above states
	if state == "play" then
		self[self.state]:unload() --we only unload the transition and pause, not the play state
		self.state = state
	elseif state == "win" or state == "lose" then
		local counts = {}
		counts.enemyCount = enemyRNG.enemyStart
		counts.timeString = convertToTimeNotation(self.play.gameTime)
		counts.playerHP = tostring(self.play.playerHP)
		self:resetStates()
		self.state = state
		self[self.state]:load(counts)
	elseif state == "new" then--win and lose set new conditions
		self.gameModifiers = self[self.state]:applyModifiers(self.gameModifiers)
		self[self.state]:unload()
		self:load("nextPlay")	
	else
		self.play.stateChange = "play"
		self.state = state
		self[self.state]:load()
	end
end

function gameState:resetStates()
	self.transition:unload()
	self.transition.stateChange = "transition"
	self.pause:unload()
	self.pause.stateChange = "pause"
	self.play:unload()
	self.play.stateChange = "play"
end

return gameState