local map = require 'engine.gameplay.map.map' --we're not doing a mapengine yet
local camera = require 'engine.lib.camera'
local player = require 'engine.gameplay.entities.player'
local enemyRNG = require 'engine.gameplay.entities.enemyrng'
local hud = require 'engine.gameplay.hud'
effects = require 'engine.gameplay.effects.effectengine'
local musicList = require 'assets.bgm.musiclist'

--this file runs all of the logic for playing
--it will load the current world via:
--bump:newWorld
--acquiring the map from the map file --> the map will load its own assets,
--then after mapping and establishing it in a bumpworld, it will
--handle logic

--for now, no maps will contain persistent data, once left behind by the player

--BIG NOTE:
--gState is being passed in update and draw because 
--during transitions and maybe pause,
--play object "displays" will undergo an update logic for JUICE
--and during "play" they will undergo actual logic

play = {
	stateChange = "play",
	index = "play",
	canPause = false,
	gameTime = 0,
	wins = 0,
	losses = 0,
	playerHP = 0,
	sfx = {text = {f = "text-9.wav", t = "static", v = 1.0,}, --path starts from "assets/sounds"
		space = {f = "opening/space-3.mp3", t = "static", v = 0.8,},
		scifi = {f = "scifi-computer-1.wav", t = "static", v = 0.5,},
		
	},
	bgm = musicList,
	audioSystem = {}
}

function play:load(game, mod, audio)
	if audio then self.audioSystem = audio end
	if game == "new" then
		audio:loadProfile(player)
		world = bump.newWorld()
		map:initialize(mod) --future mapsize update
		player:initialize()
		enemyRNG:initialize(map, mod, self.audioSystem)
		self.bgm.rand:start()
		hud:initialize(map, camera, player, enemyRNG, self)
	elseif game == "nextPlay" then
		enemyRNG:clear()
		player.hp = player.hpBase
		player.x = player.xBase
		player.y = player.yBase
		world:update(player, player.x, player.y, player.w, player.h)
		enemyRNG:initialize(map, mod, self.audioSystem)
		self.bgm.rand:resume()
		hud:initialize(map, camera, player, enemyRNG, self)	
	end
end

function play:update(dt, gState, mod)
	--var[state]:update(dt, gstate)
	if gState == "pause" then

	elseif gState == "transition" then
	
	else --gState == "play" always
		interface:handleInput(dt,player,"player")
		player:update(dt, mod)
		enemyRNG:update(dt, mod)
		damageList:resolve()
		map:update(dt, mod)
		effects:update(dt)
		camera:update(player)
		self.gameTime = self.gameTime + dt	
		self.playerHP = player.hp
		self.bgm.rand:update(dt)
		hud:update(dt, map, camera, player, enemyRNG, self)
	end
	self:handleInput(dt, gState)
	
	if enemyRNG.enemyCount < 1 or player.hp < 1 then
		print("self:checkGoals(enemyCount: "..tostring(enemyRNG.enemyCount)..", hp: "..tostring(player.hp)) 
	end
	self:checkGoals(enemyRNG.enemyCount, player.hp)
end

function play:draw(gState) --use different effects based on drawtype
	--var[state]:draw(gState)
	lg.setFont(facefont)
	camera:set()
	map:draw(camera)
	effects:draw()
	camera:unset()
	hud:draw()
end

function play:unload()
	self.gameTime = 0
end

function play:handleInput(dt, gState)
	if gState == "pause" then

	end	
	if ic.pause.isDown and gState == "play" and self.canPause then
		self.stateChange = "pause"
		self.canPause = false
	elseif not ic.pause.isDown and gState == "play" then
		self.canPause = true
	end
end

function play:checkGoals(e, hp)
	if e < 1 then
		self.stateChange = "win"
		self.wins = self.wins + 1
	elseif hp < 1 then
		self.stateChange = "lose"
		self.losses = self.losses + 1
	end
end

return play