--note:the way this works now is the interface iterates along the controls
--list *every* update. This will turn out to be very redundant.
--A better implementation might be for each control to track when
--it was last tested with updateNumber and compare, and if it hasn't
--been tested this updateNumber, re-test it. If it has, return
--its isDown member instead.

--right now it is called with ic.member.isDown
--instead, 
--function interface:isDown("control")
--	local r
--	if updateNumber != sc.member.updateNumber then
		--update member
		--r = sc.member.isDown
	--else
		--r = sc.member.isDown
	--end
--return r
--end
--doing this would require a slightly significant refactor of
--all control calls, however
--we'll do it later.

interface = {}

function interface:load()	
	self.controls = {
		
		confirm = {
			keys = {"return", "kpenter", " "},
			isDown = false,
		},
		back = {
			keys = {"escape", "backspace"},
			isDown = false,
		},
		up = {
			keys = {"up", "w"},
			isDown = false,
		},
		down = {
			keys = {"down", "s"},
			isDown = false,
		},
		left = {
			keys = {"left", "a"},
			isDown = false,
		},
		right = {
			keys = {"right", "d"},
			isDown = false,
		},
		jump = {
			keys = {" ", "up", "w"},
			isDown = false,
		},
		attack = {
			keys = {"rctrl", "lctrl"},
			isDown = false,
		},
		pause = {
			keys = {"return", "kpenter", "escape"},
			isDown = false,
		},
	}
end


function interface:testKeys()
	local sc = self.controls
	for i, v in pairs(sc) do
		t = sc[i].keys
		local r = false
		local tlen = #t
		for i=1, tlen do
			if lk.isDown(t[i]) then
				r = true
				break
			end
		end
		sc[i].isDown = r
	end
end


function interface:update(dt)
	self:testKeys()
end

function interface:handleInput(dt, env, type)
	if type == "menu" then
		self:menuControl(dt, env)
	elseif type == "player" then
		self:playerControl(dt, env)
	end
end

function interface:menuControl(dt, m)
	local mc = m.control
	local ic = self.controls
	local interact = {"confirm","back"}
	for i=1, #interact do
		local t = interact[i]
		if mc[t].u then
			if ic[t].isDown then
				mc[t].u = false
				mc[t].d = true
			end
		else
			mc[t].d = false
			if not ic[t].isDown then
				mc[t].u = true
			end
		end
	end
	
	mc.up = ic.up.isDown
	mc.down = ic.down.isDown
	mc.left = ic.left.isDown
	mc.right = ic.right.isDown
	
	m.control = mc
	
	self:resolveMenu(dt, m)
end

function interface:resolveMenu(dt, m)
	local mc = m.control
	local continue = true --hierarchy of control change priority
	if mc.back.d then
		m.m[m.state].back(m)
		continue = false
	elseif mc.confirm.d then
		m.m[m.state][m.slot].action(m, 1)
		continue = false
	end
	
	if mc.vDelay.c <= 0 and continue then
		if mc.up and not mc.down then
			m:slotChange("up", m)
			continue = false
		elseif mc.down and not mc.up then
			m:slotChange("down", m)
			continue = false
		elseif not mc.up and not mc.down then
			mc.vDelay.delay = false
		end
	end
	if mc.hDelay.c <= 0 and continue then
		if mc.right and not mc.left then
			m:optionChange(1, m)
			continue = false
		elseif mc.left and not mc.right then
			m:optionChange(-1, m)
			continue = false
		elseif not mc.left and not mc.right then
			mc.hDelay.delay = false
		end
	end
	m.control = mc
end

function interface:playerControl(dt, p)
	local sc = self.controls
	local left, right = sc.left.isDown, sc.right.isDown
	
	if left and right then --evaluate what the x velocity should be, directly
		p.input.xvel = 0
	elseif left then
		p.drawStuff.dirOffset = -p.drawStuff.dir
		p.input.direction = "left"
		p.input.xvel = -p.s
	elseif right then
		p.input.direction = "right"
		p.drawStuff.dirOffset = p.drawStuff.dir
		p.input.xvel = p.s
	else
		p.input.xvel = 0
		p.drawStuff.dirOffset = 0
	end
	
	if sc.jump.isDown then
		p.input.jumpButton = true
	else
		p.input.jumpButton = false
		p.input.jumpButtonReleased = true
	end
	
	p.attack.button = sc.attack.isDown
end

return interface