menuSystem = {
		state = "", --states: "base", "opt", "hs"
		x = 0,
		y = 0,
		h = 60,
		w = 200,
		slotX = 0,
		slotY = 0,
		slot = 1,
		columnOffset = 400,
		slotsinstate = 0,
		color = colorConstants.DOS_green,
	control = {
		index = "control",
		confirm = {d = false, u = true,},
		back = {d = false, u = true,},
		left = false,
		right = false,
		hDelay = {delay = false, c = 0, rate = 0.3, cRate = 0.3, rateMax = 0.03, init = false, mode = " ", div = 6,},
		up = false,
		down = false,
		vDelay = {delay = false, c = 0, rate = 0.3, cRate = 0.3, rateMax = 0.03, init = false, mode = " ", div = 6,},
		}
}
menuSystem.cursor = { --for now, a standardized cursor
	x = 0,
	y = 0,
	w = 350,
	h = 45,
	color = {0, 120, 0}, --explore glowing with HSLA next
	fill = "fill",
	brightening = true,
	glowRate = 2.3,
}
menuSystem.menuList = {}

--t is menuSystem.menuList[index], always
function menuSystem.cursor:update(dt, t)
	self.x = t.slotX - 20
	self.y = t.slotY - 5
	self.color[2] = colorTools:RGBglow(dt, self.glowRate, 70,100 , self.color[2], self.brightening)
	if self.color[2] >= 100 then
		self.brightening = false
	elseif self.color[2] <= 70 then
		self.brightening = true
	end
end
function menuSystem.cursor:draw()
	lg.setColor(self.color, 255)
	lg.rectangle(self.fill,self.x,self.y,self.w,self.h)
end

function menuSystem:stateSwitch(t, state)
	t.state = state
	t.slotsinstate = t.m[t.state].slots
	t.slot = 1
	t:setSlotValues(t)
	t:updateFields(t)
end

function menuSystem:update(dt, t)
	self:updateSlotCoords(t)
	t.cursor:update(dt, t)
	t:updateDelays(dt, t)
end
function menuSystem:draw(t)
	t.cursor:draw() --first, draw the cursor, then draw the menu elements
	lg.setColor(t.color,255)
	for i=1, t.slotsinstate do
		obj = t.m[t.state][i]
		lg.print(obj.text,obj.x,obj.y)
		if obj.type == "field" then
			lg.print(obj.variable, obj.x + self.columnOffset, obj.y)
		end
	end
end

function menuSystem:updateDelays(dt, t)
	local tc = t.control
	local list = {"vDelay","hDelay"}
	
	for i=1, #list do
		local d = tc[list[i]]
		if d.delay and d.init then
			d.cRate = d.cRate - dt/d.div
--			print("t.cRate: "..tostring(t.cRate).." t.rateMax: "..tostring(t.rateMax))
			if d.cRate < d.rateMax then d.cRate = d.rateMax end
		elseif d.delay and not d.init then
			d.cRate = d.rate
			d.init = true
		elseif not d.delay then
			d.cRate = d.rate
			d.init = false
		end
		
		if not ic.up.isDown and
			not ic.down.isDown and
			not ic.left.isDown and
			not ic.right.isDown then
			d.c = 0
			d.delay = false
			d.init = false
			d.mode = " "
		end
		
		if d.c > 0 then d.c = d.c - dt end
	end

end

--slot logic
function menuSystem:slotChange(dir, t)
	if dir == "up" then
		if t.slot == 1 then
			t.slot = t.slotsinstate
		else
			t.slot = t.slot - 1
		end
	elseif dir == "down" then
		if t.slot == t.slotsinstate then
			t.slot = 1
		else
			t.slot = t.slot + 1
		end
	end
	t:updateSlotCoords(t)
	
	local tcv = t.control.vDelay
	
	if tcv.mode ~= dir then
		tcv.mode = dir
		tcv.init = false
		tcv.c = tcv.rate
		tcv.delay = true
	else
		tcv.c = tcv.cRate
		tcv.delay = true
	end
end
function menuSystem:getSlotCoords(t, s)
	s = s or 1
	
	local x
	local y
	
	x = t.x
	y = t.y + (t.h * s)
	return x, y

end
function menuSystem:updateSlotCoords(t)
	t.slotX, t.slotY = t:getSlotCoords(t, t.slot)
end
function menuSystem:setSlotValues(t)
	for i=1,t.slotsinstate do
		t.m[t.state][i].x, t.m[t.state][i].y = self:getSlotCoords(t, i)
	end
end

function menuSystem:optionChange(dir, t)
	local s = t.m[t.state][t.slot]
	
	if s.type == "field" then
		s.action(t, dir)
	end	
	local tch = t.control.hDelay
	if tch.mode ~= dir then
		tch.mode = dir
		tch.init = false
		tch.c = tch.rate
		tch.delay = true
	else
		tch.c = tch.cRate
		tch.delay = true
	end
end
function menuSystem:updateFields(t)
	for i=1, t.slotsinstate do
		obj = t.m[t.state][i]
		if obj.type == "field" then
			g = obj.globalField
			obj.variable = globals[g]
		end
	end
end
function menuSystem:center(t)
	t.x = lg.getWidth()/2 - t.w
	t.y = lg.getHeight()/2
end


function menuSystem:new(menu, m)
	local n = self
	n.control.confirm.u = false
	n.control.back.u = false
	n.name = (menu.name.." subsystem")
	n.m = m
	local i = #self.menuList + 1
	menu.instance = i
	self.menuList[i] = n
end
return menuSystem